import requests
import json
import datetime
import math


def get_start_datetime(api_token):
    toggl = requests.get(
        "https://api.track.toggl.com/api/v8/me", auth=(api_token, "api_token")
    )
    data = json.loads(toggl.text)
    start_time = data["data"]["workspaces"][0]["at"]
    start_time = datetime.datetime.strptime(start_time, "%Y-%m-%dT%H:%M:%S+00:00")
    return start_time.date()


def get_workspaces(api_token):
    workspaces = []
    response = requests.get('https://api.track.toggl.com/api/v8/workspaces', auth=(api_token, 'api_token'))
    workspaces.append(json.loads(response.text))
    return workspaces
    

def get_projects(api_token):
    projects = []
    workspaces = get_workspaces(api_token)
    for workspace in workspaces[0]:
        response = requests.get(f'https://api.track.toggl.com/api/v8/workspaces/{workspace["id"]}/projects', auth=(api_token, 'api_token'))
        project = json.loads(response.text)
        if project:
            projects.append(project)
    return projects


def get_time_entries(api_token, days=100):
    start_date = get_start_datetime(api_token)
    today = datetime.date.today()
    cycles = math.ceil((today - start_date).days / days)

    data = []

    for cycle in range(cycles):
        _start_date = (start_date + datetime.timedelta(days=days) * cycle).strftime(
            "%Y-%m-%dT00:00:00-00:00"
        )
        _end_date = (start_date + datetime.timedelta(days=days) * (cycle + 1)).strftime(
            "%Y-%m-%dT00:00:00-00:00"
        )
        params = (
            ("start_date", _start_date),
            ("end_date", _end_date),
        )
        response = requests.get(
            "https://api.track.toggl.com/api/v8/time_entries",
            params=params,
            auth=(api_token, "api_token"),
        )
        data.append(json.loads(response.text))
    return data




def save_items(items, table, db):
    for item in items:
        data = item
        db[table].insert_all(data, pk="id", alter=True, replace=True)


def transform(item):
    for key in (
        "id",
        "pid",
        "tid",
        "uid",
        "dur",
        "billable",
    ):
        if key in item:
            item[key] = int(item[key])



import click
import json
import sqlite_utils
from . import utils


@click.group()
@click.version_option()
def cli():
    "Save Toggl data to a SQLite database"


@cli.command()
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to save tokens to, defaults to auth.json",
)
def auth(auth):
    "Save authentication credentials to a JSON file"
    click.echo("Visit this page and sign in with your Toggl account:\n")
    click.echo("https://track.toggl.com/profile")
    api_token = input(
        "Once you have signed in there, copy your API Toekn at the bottom of the page, paste it here, and press <enter>: "
    )
    # Now exchange the request_token for an access_token

    open(auth, "w").write(
        json.dumps(
            {
                "api_token": api_token,
            },
            indent=4,
        )
        + "\n"
    )
    click.echo("Authentication tokens written to {}".format(auth))


@cli.command()
@click.argument(
    "db_path",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    required=True,
)
@click.option(
    "-a",
    "--auth",
    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
    default="auth.json",
    help="Path to auth tokens, defaults to auth.json",
)
@click.option("-t", "--type", default=["time_entries", "workspaces", "projects"], required=True, multiple=True)
def fetch(db_path, auth, type):
    "Save Toggl data to a SQLite database"
    auth = json.load(open(auth))
    db = sqlite_utils.Database(db_path)
    if "time_entries" in type:
        time_entries = utils.get_time_entries(api_token=auth["api_token"])
        utils.save_items(time_entries, "time_entries" ,db)
    if "workspaces" in type:
        workspaces = utils.get_workspaces(api_token=auth["api_token"])
        utils.save_items(workspaces, "workspaces", db)
    if "projects" in type:
        projects = utils.get_projects(api_token=auth["api_token"])
        utils.save_items(projects, "projects" ,db)



from setuptools import setup
import os

VERSION = "0.1.6"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="toggl-to-sqlite",
    description="Create a SQLite database containing data from your Toggl account",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Ryan Cheley",
    url="https://github.com/ryancheley/toggle-to-sqlite",
    license="Apache License, Version 2.0",
    version=VERSION,
    packages=["toggl_to_sqlite"],
    entry_points="""
        [console_scripts]
        toggl-to-sqlite=toggl_to_sqlite.cli:cli
    """,
    install_requires=["sqlite-utils>=2.4.4", "click", "requests"],
    extras_require={"test": ["pytest"]},
    tests_require=["toggl-to-sqlite[test]"],
)
